@@ -1,7 +1,8 @@
 use std::num::NonZeroU32;
 
 use ruff_python_ast as ast;
-use ruff_python_ast::visitor::preorder::PreorderVisitor;
+use ruff_python_ast::visitor::source_order::SourceOrderVisitor;
+use ruff_python_ast::AstNode;
 
 use crate::ast_ids::{NodeKey, TypedNodeKey};
 use crate::cache::KeyValueCache;
@@ -11,16 +12,19 @@ use crate::module::Module;
 use crate::module::ModuleName;
 use crate::parse::parse;
 use crate::Name;
+pub(crate) use definitions::Definition;
+use definitions::{ImportDefinition, ImportFromDefinition};
+pub(crate) use flow_graph::ConstrainedDefinition;
 use flow_graph::{FlowGraph, FlowGraphBuilder, FlowNodeId, ReachableDefinitionsIterator};
+use ruff_index::{newtype_index, IndexVec};
+use rustc_hash::FxHashMap;
 use std::ops::{Deref, DerefMut};
 use std::sync::Arc;
-pub(crate) use symbol_table::{Definition, Dependency, SymbolId};
-use symbol_table::{
-    ImportDefinition, ImportFromDefinition, ScopeId, ScopeKind, SymbolFlags, SymbolTable,
-    SymbolTableBuilder,
-};
+pub(crate) use symbol_table::{Dependency, SymbolId};
+use symbol_table::{ScopeId, ScopeKind, SymbolFlags, SymbolTable, SymbolTableBuilder};
 pub(crate) use types::{infer_definition_type, infer_symbol_public_type, Type, TypeStore};
 
+mod definitions;
 mod flow_graph;
 mod symbol_table;
 mod types;
@@ -49,6 +53,9 @@ pub fn resolve_global_symbol(
     Ok(Some(GlobalSymbolId { file_id, symbol_id }))
 }
 
+#[newtype_index]
+pub struct ExpressionId;
+
 #[derive(Copy, Clone, Debug, PartialEq, Eq)]
 pub struct GlobalSymbolId {
     pub(crate) file_id: FileId,
@@ -59,6 +66,8 @@ pub struct GlobalSymbolId {
 pub struct SemanticIndex {
     symbol_table: SymbolTable,
     flow_graph: FlowGraph,
+    expressions: FxHashMap<NodeKey, ExpressionId>,
+    expressions_by_id: IndexVec<ExpressionId, NodeKey>,
 }
 
 impl SemanticIndex {
@@ -71,12 +80,25 @@ impl SemanticIndex {
                 scope_id: root_scope_id,
                 current_flow_node_id: FlowGraph::start(),
             }],
+            expressions: FxHashMap::default(),
+            expressions_by_id: IndexVec::default(),
             current_definition: None,
         };
         indexer.visit_body(&module.body);
         indexer.finish()
     }
 
+    fn resolve_expression_id<'a>(
+        &self,
+        ast: &'a ast::ModModule,
+        expression_id: ExpressionId,
+    ) -> ast::AnyNodeRef<'a> {
+        let node_key = self.expressions_by_id[expression_id];
+        node_key
+            .resolve(ast.as_any_node_ref())
+            .expect("node to resolve")
+    }
+
     /// Return an iterator over all definitions of `symbol_id` reachable from `use_expr`. The value
     /// of `symbol_id` in `use_expr` must originate from one of the iterated definitions (or from
     /// an external reassignment of the name outside of this scope).
@@ -85,13 +107,18 @@ impl SemanticIndex {
         symbol_id: SymbolId,
         use_expr: &ast::Expr,
     ) -> ReachableDefinitionsIterator {
+        let expression_id = self.expression_id(use_expr);
         ReachableDefinitionsIterator::new(
             &self.flow_graph,
             symbol_id,
-            self.flow_graph.for_expr(use_expr),
+            self.flow_graph.for_expr(expression_id),
         )
     }
 
+    pub fn expression_id(&self, expression: &ast::Expr) -> ExpressionId {
+        self.expressions[&NodeKey::from_node(expression.into())]
+    }
+
     pub fn symbol_table(&self) -> &SymbolTable {
         &self.symbol_table
     }
@@ -110,18 +137,24 @@ struct SemanticIndexer {
     scopes: Vec<ScopeState>,
     /// the definition whose target(s) we are currently walking
     current_definition: Option<Definition>,
+    expressions: FxHashMap<NodeKey, ExpressionId>,
+    expressions_by_id: IndexVec<ExpressionId, NodeKey>,
 }
 
 impl SemanticIndexer {
-    pub(crate) fn finish(self) -> SemanticIndex {
+    pub(crate) fn finish(mut self) -> SemanticIndex {
         let SemanticIndexer {
             flow_graph_builder,
             symbol_table_builder,
             ..
         } = self;
+        self.expressions.shrink_to_fit();
+        self.expressions_by_id.shrink_to_fit();
         SemanticIndex {
             flow_graph: flow_graph_builder.finish(),
             symbol_table: symbol_table_builder.finish(),
+            expressions: self.expressions,
+            expressions_by_id: self.expressions_by_id,
         }
     }
 
@@ -197,6 +230,15 @@ impl SemanticIndexer {
             .record_scope_for_node(node_key, scope_id);
     }
 
+    fn insert_constraint(&mut self, expr: &ast::Expr) {
+        let node_key = NodeKey::from_node(expr.into());
+        let expression_id = self.expressions[&node_key];
+        let constraint = self
+            .flow_graph_builder
+            .add_constraint(self.current_flow_node(), expression_id);
+        self.set_current_flow_node(constraint);
+    }
+
     fn with_type_params(
         &mut self,
         name: &str,
@@ -224,25 +266,80 @@ impl SemanticIndexer {
     }
 }
 
-impl PreorderVisitor<'_> for SemanticIndexer {
+impl SourceOrderVisitor<'_> for SemanticIndexer {
     fn visit_expr(&mut self, expr: &ast::Expr) {
-        if let ast::Expr::Name(ast::ExprName { id, ctx, .. }) = expr {
-            let flags = match ctx {
-                ast::ExprContext::Load => SymbolFlags::IS_USED,
-                ast::ExprContext::Store => SymbolFlags::IS_DEFINED,
-                ast::ExprContext::Del => SymbolFlags::IS_DEFINED,
-                ast::ExprContext::Invalid => SymbolFlags::empty(),
-            };
-            self.add_or_update_symbol(id, flags);
-            if flags.contains(SymbolFlags::IS_DEFINED) {
-                if let Some(curdef) = self.current_definition.clone() {
-                    self.add_or_update_symbol_with_def(id, curdef);
+        let node_key = NodeKey::from_node(expr.into());
+        let expression_id = self.expressions_by_id.push(node_key);
+
+        debug_assert_eq!(
+            expression_id,
+            self.flow_graph_builder
+                .record_expr(self.current_flow_node())
+        );
+
+        debug_assert_eq!(
+            expression_id,
+            self.symbol_table_builder
+                .record_expression(self.cur_scope())
+        );
+
+        self.expressions.insert(node_key, expression_id);
+
+        match expr {
+            ast::Expr::Name(ast::ExprName { id, ctx, .. }) => {
+                let flags = match ctx {
+                    ast::ExprContext::Load => SymbolFlags::IS_USED,
+                    ast::ExprContext::Store => SymbolFlags::IS_DEFINED,
+                    ast::ExprContext::Del => SymbolFlags::IS_DEFINED,
+                    ast::ExprContext::Invalid => SymbolFlags::empty(),
+                };
+                self.add_or_update_symbol(id, flags);
+                if flags.contains(SymbolFlags::IS_DEFINED) {
+                    if let Some(curdef) = self.current_definition.clone() {
+                        self.add_or_update_symbol_with_def(id, curdef);
+                    }
                 }
+                ast::visitor::source_order::walk_expr(self, expr);
+            }
+            ast::Expr::Named(node) => {
+                debug_assert!(self.current_definition.is_none());
+                self.current_definition =
+                    Some(Definition::NamedExpr(TypedNodeKey::from_node(node)));
+                // TODO walrus in comprehensions is implicitly nonlocal
+                self.visit_expr(&node.target);
+                self.current_definition = None;
+                self.visit_expr(&node.value);
+            }
+            ast::Expr::If(ast::ExprIf {
+                body, test, orelse, ..
+            }) => {
+                // TODO detect statically known truthy or falsy test (via type inference, not naive
+                // AST inspection, so we can't simplify here, need to record test expression in CFG
+                // for later checking)
+
+                self.visit_expr(test);
+
+                let if_branch = self.flow_graph_builder.add_branch(self.current_flow_node());
+
+                self.set_current_flow_node(if_branch);
+                self.insert_constraint(test);
+                self.visit_expr(body);
+
+                let post_body = self.current_flow_node();
+
+                self.set_current_flow_node(if_branch);
+                self.visit_expr(orelse);
+
+                let post_else = self
+                    .flow_graph_builder
+                    .add_phi(self.current_flow_node(), post_body);
+
+                self.set_current_flow_node(post_else);
+            }
+            _ => {
+                ast::visitor::source_order::walk_expr(self, expr);
             }
         }
-        self.flow_graph_builder
-            .record_expr(expr, self.current_flow_node());
-        ast::visitor::preorder::walk_expr(self, expr);
     }
 
     fn visit_stmt(&mut self, stmt: &ast::Stmt) {
@@ -367,12 +464,20 @@ impl PreorderVisitor<'_> for SemanticIndexer {
             }
             ast::Stmt::Assign(node) => {
                 debug_assert!(self.current_definition.is_none());
+                self.visit_expr(&node.value);
                 self.current_definition =
                     Some(Definition::Assignment(TypedNodeKey::from_node(node)));
-                ast::visitor::preorder::walk_stmt(self, stmt);
+                for expr in &node.targets {
+                    self.visit_expr(expr);
+                }
+
                 self.current_definition = None;
             }
             ast::Stmt::If(node) => {
+                // TODO detect statically known truthy or falsy test (via type inference, not naive
+                // AST inspection, so we can't simplify here, need to record test expression in CFG
+                // for later checking)
+
                 // we visit the if "test" condition first regardless
                 self.visit_expr(&node.test);
 
@@ -381,6 +486,7 @@ impl PreorderVisitor<'_> for SemanticIndexer {
 
                 // visit the body of the `if` clause
                 self.set_current_flow_node(if_branch);
+                self.insert_constraint(&node.test);
                 self.visit_body(&node.body);
 
                 // Flow node for the last if/elif condition branch; represents the "no branch
@@ -399,13 +505,15 @@ impl PreorderVisitor<'_> for SemanticIndexer {
                 let mut last_branch_is_else = false;
 
                 for clause in &node.elif_else_clauses {
-                    if clause.test.is_some() {
+                    if let Some(test) = &clause.test {
+                        self.visit_expr(test);
                         // This is an elif clause. Create a new branch node. Its predecessor is the
                         // previous branch node, because we can only take one branch in an entire
                         // if/elif/else chain, so if we take this branch, it can only be because we
                         // didn't take the previous one.
                         prior_branch = self.flow_graph_builder.add_branch(prior_branch);
                         self.set_current_flow_node(prior_branch);
+                        self.insert_constraint(test);
                     } else {
                         // This is an else clause. No need to create a branch node; there's no
                         // branch here, if we haven't taken any previous branch, we definitely go
@@ -435,7 +543,7 @@ impl PreorderVisitor<'_> for SemanticIndexer {
                 self.set_current_flow_node(post_prior_clause);
             }
             _ => {
-                ast::visitor::preorder::walk_stmt(self, stmt);
+                ast::visitor::source_order::walk_stmt(self, stmt);
             }
         }
     }
@@ -727,7 +835,10 @@ mod tests {
         let ast::Stmt::Expr(ast::StmtExpr { value: x_use, .. }) = &ast.body[1] else {
             panic!("should be an expr")
         };
-        let x_defs: Vec<_> = index.reachable_definitions(x_sym, x_use).collect();
+        let x_defs: Vec<_> = index
+            .reachable_definitions(x_sym, x_use)
+            .map(|constrained_definition| constrained_definition.definition)
+            .collect();
         assert_eq!(x_defs.len(), 1);
         let Definition::Assignment(node_key) = &x_defs[0] else {
             panic!("def should be an assignment")
@@ -744,4 +855,29 @@ mod tests {
         };
         assert_eq!(*num, 1);
     }
+
+    #[test]
+    fn expression_scope() {
+        let parsed = parse("x = 1;\ndef test():\n  y = 4");
+        let ast = parsed.syntax();
+        let index = SemanticIndex::from_ast(ast);
+        let table = &index.symbol_table;
+
+        let x_sym = table
+            .root_symbol_by_name("x")
+            .expect("x symbol should exist");
+
+        let x_stmt = ast.body[0].as_assign_stmt().unwrap();
+
+        let x_id = index.expression_id(&x_stmt.targets[0]);
+
+        assert_eq!(table.scope_of_expression(x_id).kind(), ScopeKind::Module);
+        assert_eq!(table.scope_id_of_expression(x_id), x_sym.scope_id());
+
+        let def = ast.body[1].as_function_def_stmt().unwrap();
+        let y_stmt = def.body[0].as_assign_stmt().unwrap();
+        let y_id = index.expression_id(&y_stmt.targets[0]);
+
+        assert_eq!(table.scope_of_expression(y_id).kind(), ScopeKind::Function);
+    }
 }
