@@ -8,6 +8,7 @@ use crate::semantic::{
 };
 use crate::{FxDashMap, FxIndexSet, Name};
 use ruff_index::{newtype_index, IndexVec};
+use ruff_python_ast as ast;
 use rustc_hash::FxHashMap;
 
 pub(crate) mod infer;
@@ -17,7 +18,7 @@ pub(crate) use infer::{infer_definition_type, infer_symbol_public_type};
 /// unique ID for a type
 #[derive(Copy, Clone, Debug, PartialEq, Eq, Hash)]
 pub enum Type {
-    /// the dynamic or gradual type: a statically-unknown set of values
+    /// the dynamic type: a statically-unknown set of values
     Any,
     /// the empty set of values
     Never,
@@ -26,6 +27,8 @@ pub enum Type {
     Unknown,
     /// name is not bound to any value
     Unbound,
+    /// the None object (TODO remove this in favor of Instance(types.NoneType)
+    None,
     /// a specific function object
     Function(FunctionTypeId),
     /// a specific module object
@@ -55,10 +58,11 @@ impl Type {
 
     pub fn get_member(&self, db: &dyn SemanticDb, name: &Name) -> QueryResult<Option<Type>> {
         match self {
-            Type::Any => todo!("attribute lookup on Any type"),
+            Type::Any => Ok(Some(Type::Any)),
             Type::Never => todo!("attribute lookup on Never type"),
-            Type::Unknown => todo!("attribute lookup on Unknown type"),
+            Type::Unknown => Ok(Some(Type::Unknown)),
             Type::Unbound => todo!("attribute lookup on Unbound type"),
+            Type::None => todo!("attribute lookup on None type"),
             Type::Function(_) => todo!("attribute lookup on Function type"),
             Type::Module(module_id) => module_id.get_member(db, name),
             Type::Class(class_id) => class_id.get_class_member(db, name),
@@ -85,6 +89,56 @@ impl Type {
             }
         }
     }
+
+    // when this is fully fleshed out, it will use the db arg and may return QueryError
+    #[allow(clippy::unnecessary_wraps)]
+    pub fn resolve_bin_op(
+        &self,
+        _db: &dyn SemanticDb,
+        op: ast::Operator,
+        right_ty: Type,
+    ) -> QueryResult<Type> {
+        match self {
+            Type::Any => Ok(Type::Any),
+            Type::Unknown => Ok(Type::Unknown),
+            Type::IntLiteral(n) => {
+                match right_ty {
+                    Type::IntLiteral(m) => {
+                        match op {
+                            ast::Operator::Add => Ok(n
+                                .checked_add(m)
+                                .map(Type::IntLiteral)
+                                // TODO builtins.int
+                                .unwrap_or(Type::Unknown)),
+                            ast::Operator::Sub => Ok(n
+                                .checked_sub(m)
+                                .map(Type::IntLiteral)
+                                // TODO builtins.int
+                                .unwrap_or(Type::Unknown)),
+                            ast::Operator::Mult => Ok(n
+                                .checked_mul(m)
+                                .map(Type::IntLiteral)
+                                // TODO builtins.int
+                                .unwrap_or(Type::Unknown)),
+                            ast::Operator::Div => Ok(n
+                                .checked_div(m)
+                                .map(Type::IntLiteral)
+                                // TODO builtins.int
+                                .unwrap_or(Type::Unknown)),
+                            ast::Operator::Mod => Ok(n
+                                .checked_rem(m)
+                                .map(Type::IntLiteral)
+                                // TODO division by zero error
+                                .unwrap_or(Type::Unknown)),
+                            _ => todo!("complete binop op support for IntLiteral"),
+                        }
+                    }
+                    _ => todo!("complete binop right_ty support for IntLiteral"),
+                }
+            }
+            _ => todo!("complete binop support"),
+        }
+    }
 }
 
 impl From<FunctionTypeId> for Type {
@@ -159,7 +213,7 @@ impl TypeStore {
         self.modules.get(&file_id)
     }
 
-    fn add_function(
+    fn add_function_type(
         &self,
         file_id: FileId,
         name: &str,
@@ -171,7 +225,18 @@ impl TypeStore {
             .add_function(name, symbol_id, scope_id, decorators)
     }
 
-    fn add_class(
+    fn add_function(
+        &self,
+        file_id: FileId,
+        name: &str,
+        symbol_id: SymbolId,
+        scope_id: ScopeId,
+        decorators: Vec<Type>,
+    ) -> Type {
+        Type::Function(self.add_function_type(file_id, name, symbol_id, scope_id, decorators))
+    }
+
+    fn add_class_type(
         &self,
         file_id: FileId,
         name: &str,
@@ -182,11 +247,35 @@ impl TypeStore {
             .add_class(name, scope_id, bases)
     }
 
-    fn add_union(&self, file_id: FileId, elems: &[Type]) -> UnionTypeId {
+    fn add_class(&self, file_id: FileId, name: &str, scope_id: ScopeId, bases: Vec<Type>) -> Type {
+        Type::Class(self.add_class_type(file_id, name, scope_id, bases))
+    }
+
+    /// add "raw" union type with exactly given elements
+    fn add_union_type(&self, file_id: FileId, elems: &[Type]) -> UnionTypeId {
         self.add_or_get_module(file_id).add_union(elems)
     }
 
-    fn add_intersection(
+    /// add union with normalization; may not return a `UnionType`
+    fn add_union(&self, file_id: FileId, elems: &[Type]) -> Type {
+        let mut flattened = Vec::with_capacity(elems.len());
+        for ty in elems {
+            match ty {
+                Type::Union(union_id) => flattened.extend(union_id.elements(self)),
+                _ => flattened.push(*ty),
+            }
+        }
+        // TODO don't add identical unions
+        // TODO de-duplicate union elements
+        match flattened[..] {
+            [] => Type::Never,
+            [ty] => ty,
+            _ => Type::Union(self.add_union_type(file_id, &flattened)),
+        }
+    }
+
+    /// add "raw" intersection type with exactly given elements
+    fn add_intersection_type(
         &self,
         file_id: FileId,
         positive: &[Type],
@@ -196,6 +285,38 @@ impl TypeStore {
             .add_intersection(positive, negative)
     }
 
+    /// add intersection with normalization; may not return an `IntersectionType`
+    fn add_intersection(&self, file_id: FileId, positive: &[Type], negative: &[Type]) -> Type {
+        let mut pos_flattened = Vec::with_capacity(positive.len());
+        let mut neg_flattened = Vec::with_capacity(negative.len());
+        for ty in positive {
+            match ty {
+                Type::Intersection(intersection_id) => {
+                    pos_flattened.extend(intersection_id.positive(self));
+                    neg_flattened.extend(intersection_id.negative(self));
+                }
+                _ => pos_flattened.push(*ty),
+            }
+        }
+        for ty in negative {
+            match ty {
+                Type::Intersection(intersection_id) => {
+                    pos_flattened.extend(intersection_id.negative(self));
+                    neg_flattened.extend(intersection_id.positive(self));
+                }
+                _ => neg_flattened.push(*ty),
+            }
+        }
+        // TODO don't add identical intersections
+        // TODO deduplicate intersection elements
+        // TODO maintain DNF form (union of intersections)
+        match (&pos_flattened[..], &neg_flattened[..]) {
+            ([], []) => Type::Any, // TODO should be object
+            ([ty], []) => *ty,
+            (pos, neg) => Type::Intersection(self.add_intersection_type(file_id, pos, neg)),
+        }
+    }
+
     fn get_function(&self, id: FunctionTypeId) -> FunctionTypeRef {
         FunctionTypeRef {
             module_store: self.get_module(id.file_id),
@@ -469,12 +590,31 @@ pub struct UnionTypeId {
     union_id: ModuleUnionTypeId,
 }
 
+impl UnionTypeId {
+    pub fn elements(self, type_store: &TypeStore) -> Vec<Type> {
+        let union = type_store.get_union(self);
+        union.elements.iter().copied().collect()
+    }
+}
+
 #[derive(Copy, Clone, Debug, Hash, Eq, PartialEq)]
 pub struct IntersectionTypeId {
     file_id: FileId,
     intersection_id: ModuleIntersectionTypeId,
 }
 
+impl IntersectionTypeId {
+    pub fn positive(self, type_store: &TypeStore) -> Vec<Type> {
+        let intersection = type_store.get_intersection(self);
+        intersection.positive.iter().copied().collect()
+    }
+
+    pub fn negative(self, type_store: &TypeStore) -> Vec<Type> {
+        let intersection = type_store.get_intersection(self);
+        intersection.negative.iter().copied().collect()
+    }
+}
+
 #[newtype_index]
 struct ModuleFunctionTypeId;
 
@@ -600,6 +740,7 @@ impl std::fmt::Display for DisplayType<'_> {
             Type::Never => f.write_str("Never"),
             Type::Unknown => f.write_str("Unknown"),
             Type::Unbound => f.write_str("Unbound"),
+            Type::None => f.write_str("None"),
             Type::Module(module_id) => {
                 // NOTE: something like this?: "<module 'module-name' from 'path-from-fileid'>"
                 todo!("{module_id:?}")
@@ -681,16 +822,42 @@ pub(crate) struct UnionType {
 
 impl UnionType {
     fn display(&self, f: &mut std::fmt::Formatter<'_>, store: &TypeStore) -> std::fmt::Result {
-        f.write_str("(")?;
+        let (int_literals, other_types): (Vec<Type>, Vec<Type>) = self
+            .elements
+            .iter()
+            .copied()
+            .partition(|ty| matches!(ty, Type::IntLiteral(_)));
         let mut first = true;
-        for ty in &self.elements {
+        if !int_literals.is_empty() {
+            f.write_str("Literal[")?;
+            let mut nums: Vec<i64> = int_literals
+                .into_iter()
+                .filter_map(|ty| {
+                    if let Type::IntLiteral(n) = ty {
+                        Some(n)
+                    } else {
+                        None
+                    }
+                })
+                .collect();
+            nums.sort_unstable();
+            for num in nums {
+                if !first {
+                    f.write_str(", ")?;
+                }
+                write!(f, "{num}")?;
+                first = false;
+            }
+            f.write_str("]")?;
+        }
+        for ty in other_types {
             if !first {
                 f.write_str(" | ")?;
             };
             first = false;
             write!(f, "{}", ty.display(store))?;
         }
-        f.write_str(")")
+        Ok(())
     }
 }
 
@@ -710,7 +877,6 @@ pub(crate) struct IntersectionType {
 
 impl IntersectionType {
     fn display(&self, f: &mut std::fmt::Formatter<'_>, store: &TypeStore) -> std::fmt::Result {
-        f.write_str("(")?;
         let mut first = true;
         for (neg, ty) in self
             .positive
@@ -727,7 +893,7 @@ impl IntersectionType {
             };
             write!(f, "{}", ty.display(store))?;
         }
-        f.write_str(")")
+        Ok(())
     }
 }
 
@@ -738,15 +904,68 @@ mod tests {
 
     use crate::files::Files;
     use crate::semantic::symbol_table::SymbolTableBuilder;
-    use crate::semantic::{SymbolFlags, SymbolTable, TypeStore};
+    use crate::semantic::{FileId, ScopeId, SymbolFlags, SymbolTable, TypeStore};
     use crate::FxIndexSet;
 
+    struct TestCase {
+        store: TypeStore,
+        files: Files,
+        file_id: FileId,
+        root_scope: ScopeId,
+    }
+
+    fn create_test() -> TestCase {
+        let files = Files::default();
+        let file_id = files.intern(Path::new("/foo"));
+        TestCase {
+            store: TypeStore::default(),
+            files,
+            file_id,
+            root_scope: SymbolTable::root_scope_id(),
+        }
+    }
+
+    fn assert_union_elements(store: &TypeStore, union: Type, elements: &[Type]) {
+        let Type::Union(union_id) = union else {
+            panic!("should be a union")
+        };
+
+        assert_eq!(
+            store.get_union(union_id).elements,
+            elements.iter().copied().collect::<FxIndexSet<_>>()
+        );
+    }
+
+    fn assert_intersection_elements(
+        store: &TypeStore,
+        intersection: Type,
+        positive: &[Type],
+        negative: &[Type],
+    ) {
+        let Type::Intersection(intersection_id) = intersection else {
+            panic!("should be a intersection")
+        };
+
+        assert_eq!(
+            store.get_intersection(intersection_id).positive,
+            positive.iter().copied().collect::<FxIndexSet<_>>()
+        );
+        assert_eq!(
+            store.get_intersection(intersection_id).negative,
+            negative.iter().copied().collect::<FxIndexSet<_>>()
+        );
+    }
+
     #[test]
     fn add_class() {
-        let store = TypeStore::default();
-        let files = Files::default();
-        let file_id = files.intern(Path::new("/foo"));
-        let id = store.add_class(file_id, "C", SymbolTable::root_scope_id(), Vec::new());
+        let TestCase {
+            store,
+            file_id,
+            root_scope,
+            ..
+        } = create_test();
+
+        let id = store.add_class_type(file_id, "C", root_scope, Vec::new());
         assert_eq!(store.get_class(id).name(), "C");
         let inst = Type::Instance(id);
         assert_eq!(format!("{}", inst.display(&store)), "C");
@@ -754,9 +973,13 @@ mod tests {
 
     #[test]
     fn add_function() {
-        let store = TypeStore::default();
-        let files = Files::default();
-        let file_id = files.intern(Path::new("/foo"));
+        let TestCase {
+            store,
+            file_id,
+            root_scope,
+            ..
+        } = create_test();
+
         let mut builder = SymbolTableBuilder::new();
         let func_symbol = builder.add_or_update_symbol(
             SymbolTable::root_scope_id(),
@@ -765,11 +988,11 @@ mod tests {
         );
         builder.finish();
 
-        let id = store.add_function(
+        let id = store.add_function_type(
             file_id,
             "func",
             func_symbol,
-            SymbolTable::root_scope_id(),
+            root_scope,
             vec![Type::Unknown],
         );
         assert_eq!(store.get_function(id).name(), "func");
@@ -780,44 +1003,117 @@ mod tests {
 
     #[test]
     fn add_union() {
-        let store = TypeStore::default();
-        let files = Files::default();
-        let file_id = files.intern(Path::new("/foo"));
-        let c1 = store.add_class(file_id, "C1", SymbolTable::root_scope_id(), Vec::new());
-        let c2 = store.add_class(file_id, "C2", SymbolTable::root_scope_id(), Vec::new());
+        let TestCase {
+            store,
+            file_id,
+            root_scope,
+            ..
+        } = create_test();
+
+        let c1 = store.add_class_type(file_id, "C1", root_scope, Vec::new());
+        let c2 = store.add_class_type(file_id, "C2", root_scope, Vec::new());
         let elems = vec![Type::Instance(c1), Type::Instance(c2)];
-        let id = store.add_union(file_id, &elems);
-        assert_eq!(
-            store.get_union(id).elements,
-            elems.into_iter().collect::<FxIndexSet<_>>()
-        );
+        let id = store.add_union_type(file_id, &elems);
         let union = Type::Union(id);
-        assert_eq!(format!("{}", union.display(&store)), "(C1 | C2)");
+
+        assert_union_elements(&store, union, &elems);
+        assert_eq!(format!("{}", union.display(&store)), "C1 | C2");
     }
 
     #[test]
     fn add_intersection() {
-        let store = TypeStore::default();
-        let files = Files::default();
-        let file_id = files.intern(Path::new("/foo"));
-        let c1 = store.add_class(file_id, "C1", SymbolTable::root_scope_id(), Vec::new());
-        let c2 = store.add_class(file_id, "C2", SymbolTable::root_scope_id(), Vec::new());
-        let c3 = store.add_class(file_id, "C3", SymbolTable::root_scope_id(), Vec::new());
+        let TestCase {
+            store,
+            file_id,
+            root_scope,
+            ..
+        } = create_test();
+
+        let c1 = store.add_class_type(file_id, "C1", root_scope, Vec::new());
+        let c2 = store.add_class_type(file_id, "C2", root_scope, Vec::new());
+        let c3 = store.add_class_type(file_id, "C3", root_scope, Vec::new());
         let pos = vec![Type::Instance(c1), Type::Instance(c2)];
         let neg = vec![Type::Instance(c3)];
-        let id = store.add_intersection(file_id, &pos, &neg);
-        assert_eq!(
-            store.get_intersection(id).positive,
-            pos.into_iter().collect::<FxIndexSet<_>>()
-        );
-        assert_eq!(
-            store.get_intersection(id).negative,
-            neg.into_iter().collect::<FxIndexSet<_>>()
-        );
+        let id = store.add_intersection_type(file_id, &pos, &neg);
         let intersection = Type::Intersection(id);
-        assert_eq!(
-            format!("{}", intersection.display(&store)),
-            "(C1 & C2 & ~C3)"
-        );
+
+        assert_intersection_elements(&store, intersection, &pos, &neg);
+        assert_eq!(format!("{}", intersection.display(&store)), "C1 & C2 & ~C3");
+    }
+
+    #[test]
+    fn flatten_union_zero_elements() {
+        let TestCase { store, file_id, .. } = create_test();
+
+        let ty = store.add_union(file_id, &[]);
+
+        assert!(matches!(ty, Type::Never), "{ty:?} should be Never");
+    }
+
+    #[test]
+    fn flatten_union_one_element() {
+        let TestCase { store, file_id, .. } = create_test();
+
+        let ty = store.add_union(file_id, &[Type::None]);
+
+        assert!(matches!(ty, Type::None), "{ty:?} should be None");
+    }
+
+    #[test]
+    fn flatten_nested_union() {
+        let TestCase { store, file_id, .. } = create_test();
+
+        let l1 = Type::IntLiteral(1);
+        let l2 = Type::IntLiteral(2);
+        let u1 = store.add_union(file_id, &[l1, l2]);
+        let u2 = store.add_union(file_id, &[u1, Type::None]);
+
+        assert_union_elements(&store, u2, &[l1, l2, Type::None]);
+    }
+
+    #[test]
+    fn flatten_intersection_zero_elements() {
+        let TestCase { store, file_id, .. } = create_test();
+
+        let ty = store.add_intersection(file_id, &[], &[]);
+
+        // TODO should be object, not Any
+        assert!(matches!(ty, Type::Any), "{ty:?} should be object");
+    }
+
+    #[test]
+    fn flatten_intersection_one_positive_element() {
+        let TestCase { store, file_id, .. } = create_test();
+
+        let ty = store.add_intersection(file_id, &[Type::None], &[]);
+
+        assert!(matches!(ty, Type::None), "{ty:?} should be None");
+    }
+
+    #[test]
+    fn flatten_intersection_one_negative_element() {
+        let TestCase { store, file_id, .. } = create_test();
+
+        let ty = store.add_intersection(file_id, &[], &[Type::None]);
+
+        assert_intersection_elements(&store, ty, &[], &[Type::None]);
+    }
+
+    #[test]
+    fn flatten_nested_intersection() {
+        let TestCase {
+            store,
+            file_id,
+            root_scope,
+            ..
+        } = create_test();
+
+        let c1 = Type::Instance(store.add_class_type(file_id, "C1", root_scope, vec![]));
+        let c2 = Type::Instance(store.add_class_type(file_id, "C2", root_scope, vec![]));
+        let c1sub = Type::Instance(store.add_class_type(file_id, "C1sub", root_scope, vec![c1]));
+        let i1 = store.add_intersection(file_id, &[c1, c2], &[c1sub]);
+        let i2 = store.add_intersection(file_id, &[i1, Type::None], &[]);
+
+        assert_intersection_elements(&store, i2, &[c1, c2, Type::None], &[c1sub]);
     }
 }
