@@ -1,6 +1,7 @@
-use crate::file_system::{FileSystem, FileSystemPath, FileType, Metadata, Result};
 use filetime::FileTime;
 
+use crate::file_system::{FileSystem, FileSystemPath, FileType, Metadata, Result};
+
 pub struct OsFileSystem;
 
 impl OsFileSystem {
@@ -15,6 +16,10 @@ impl OsFileSystem {
     fn permissions(_metadata: &std::fs::Metadata) -> Option<u32> {
         None
     }
+
+    pub fn snapshot(&self) -> Self {
+        Self
+    }
 }
 
 impl FileSystem for OsFileSystem {
