@@ -49,29 +49,29 @@ result = [
 
 #: E203:1:10
 if x == 4 :
-    print x, y
+    print(x, y)
     x, y = y, x
 #: E203:1:10
 if x == 4	:
-    print x, y
+    print(x, y)
     x, y = y, x
 #: E203:2:15 E702:2:16
 if x == 4:
-    print x, y ; x, y = y, x
+    print(x, y) ; x, y = y, x
 #: E203:2:15 E702:2:16
 if x == 4:
-    print x, y	; x, y = y, x
+    print(x, y)	; x, y = y, x
 #: E203:3:13
 if x == 4:
-    print x, y
+    print(x, y)
     x, y = y , x
 #: E203:3:13
 if x == 4:
-    print x, y
+    print(x, y)
     x, y = y	, x
 #: Okay
 if x == 4:
-    print x, y
+    print(x, y)
     x, y = y, x
 a[b1, :] == a[b1, ...]
 b = a[:, b1]
@@ -123,7 +123,7 @@ ham[lower + offset : upper + offset]
 ham[(lower + offset) : upper + offset]
 
 #: E203:1:19
-ham{lower + offset : upper + offset}
+{lower + offset : upper + offset}
 
 #: E203:1:19
 ham[lower + offset  : upper + offset]
