@@ -20,6 +20,7 @@ pub(crate) use non_self_return_type::*;
 pub(crate) use numeric_literal_too_long::*;
 pub(crate) use pass_in_class_body::*;
 pub(crate) use pass_statement_stub_body::*;
+pub(crate) use pre_pep570_positional_argument::*;
 pub(crate) use prefix_type_params::*;
 pub(crate) use quoted_annotation_in_stub::*;
 pub(crate) use redundant_final_literal::*;
@@ -62,6 +63,7 @@ mod non_self_return_type;
 mod numeric_literal_too_long;
 mod pass_in_class_body;
 mod pass_statement_stub_body;
+mod pre_pep570_positional_argument;
 mod prefix_type_params;
 mod quoted_annotation_in_stub;
 mod redundant_final_literal;
