@@ -687,10 +687,24 @@ pub struct ExprIf<'a> {
     orelse: Box<ComparableExpr<'a>>,
 }
 
+#[derive(Debug, PartialEq, Eq, Hash)]
+pub struct ComparableDictItem<'a> {
+    key: Option<ComparableExpr<'a>>,
+    value: ComparableExpr<'a>,
+}
+
+impl<'a> From<&'a ast::DictItem> for ComparableDictItem<'a> {
+    fn from(ast::DictItem { key, value }: &'a ast::DictItem) -> Self {
+        Self {
+            key: key.as_ref().map(ComparableExpr::from),
+            value: value.into(),
+        }
+    }
+}
+
 #[derive(Debug, PartialEq, Eq, Hash)]
 pub struct ExprDict<'a> {
-    keys: Vec<Option<ComparableExpr<'a>>>,
-    values: Vec<ComparableExpr<'a>>,
+    items: Vec<ComparableDictItem<'a>>,
 }
 
 #[derive(Debug, PartialEq, Eq, Hash)]
@@ -933,16 +947,8 @@ impl<'a> From<&'a ast::Expr> for ComparableExpr<'a> {
                 body: body.into(),
                 orelse: orelse.into(),
             }),
-            ast::Expr::Dict(ast::ExprDict {
-                keys,
-                values,
-                range: _,
-            }) => Self::Dict(ExprDict {
-                keys: keys
-                    .iter()
-                    .map(|expr| expr.as_ref().map(Into::into))
-                    .collect(),
-                values: values.iter().map(Into::into).collect(),
+            ast::Expr::Dict(ast::ExprDict { items, range: _ }) => Self::Dict(ExprDict {
+                items: items.iter().map(ComparableDictItem::from).collect(),
             }),
             ast::Expr::Set(ast::ExprSet { elts, range: _ }) => Self::Set(ExprSet {
                 elts: elts.iter().map(Into::into).collect(),
